@@ -1,9 +1,9 @@
 /* Test of fclose module.
-   Copyright (C) 2011-2017 Free Software Foundation, Inc.
+   Copyright (C) 2011-2023 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
+   the Free Software Foundation, either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -75,6 +75,7 @@ main (int argc, char **argv)
 
   /* Test that fclose() sets errno if someone else closes the stream
      fd behind the back of stdio.  */
+  #if !defined __ANDROID__ /* fdsan */
   {
     FILE *fp = fdopen (fd, "w+");
     ASSERT (fp != NULL);
@@ -83,6 +84,7 @@ main (int argc, char **argv)
     ASSERT (fclose (fp) == EOF);
     ASSERT (errno == EBADF);
   }
+  #endif
 
   /* Test that fclose() sets errno if the stream was constructed with
      an invalid file descriptor.  */
