@@ -1,9 +1,9 @@
 /* Test of iswblank() function.
-   Copyright (C) 2007-2012 Free Software Foundation, Inc.
+   Copyright (C) 2007-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -31,5 +31,10 @@ main (void)
   /* Check that the isw* functions map WEOF to 0.  */
   ASSERT (!iswblank (e));
 
+  /* Sanity check for the iswblank function.  */
+  ASSERT (iswblank (L' '));
+  ASSERT (iswblank (L'\t'));
+  ASSERT (!iswblank (L'\n'));
+
   return 0;
 }
