@@ -1,7 +1,7 @@
 #!/bin/sh
 
 # Test whether a specific Turkish locale is installed.
-: ${LOCALE_TR_UTF8=tr_TR.UTF-8}
+: "${LOCALE_TR_UTF8=tr_TR.UTF-8}"
 if test $LOCALE_TR_UTF8 = none; then
   if test -f /usr/bin/localedef; then
     echo "Skipping test: no turkish Unicode locale is installed"
@@ -12,4 +12,4 @@ if test $LOCALE_TR_UTF8 = none; then
 fi
 
 LC_ALL=$LOCALE_TR_UTF8 \
-./test-mbscasecmp${EXEEXT}
+${CHECKER} ./test-mbscasecmp${EXEEXT}
