@@ -1,11 +1,11 @@
 #!/bin/sh
 # Test suite for update-copyright.
-# Copyright (C) 2009-2012 Free Software Foundation, Inc.
+# Copyright (C) 2009-2023 Free Software Foundation, Inc.
 # This file is part of the GNUlib Library.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 diffout=`diff -u /dev/null /dev/null 2>&1`
 if test x"$diffout" = x"" && test $? -eq 0; then
@@ -28,7 +28,7 @@ PATH=$abs_aux_dir:$PATH
 export PATH
 
 TMP_BASE=update-copyright.test
-trap 'rm -f $TMP_BASE*' 0 1 2 3 15
+trap 'rm -f $TMP_BASE*' EXIT HUP INT QUIT TERM
 
 ## --------------------------------- ##
 ## Skip if user does not have perl.  ##
@@ -37,9 +37,23 @@ trap 'rm -f $TMP_BASE*' 0 1 2 3 15
 TMP=$TMP_BASE
 s=$TMP-script
 cat <<\EOF > $s
-eval '(exit $?0)' && eval 'exec perl -wS -0777 -pi "$0" ${1+"$@"}'
-  & eval 'exec perl -wS -0777 -pi "$0" $argv:q'
-    if 0;
+#!/bin/sh
+#! -*-perl-*-
+# This is a prologue that allows to run a perl script as an executable
+# on systems that are compliant to a POSIX version before POSIX:2017.
+# On such systems, the usual invocation of an executable through execlp()
+# or execvp() fails with ENOEXEC if it is a script that does not start
+# with a #! line.  The script interpreter mentioned in the #! line has
+# to be /bin/sh, because on GuixSD systems that is the only program that
+# has a fixed file name.  The second line is essential for perl and is
+# also useful for editing this file in Emacs.  The next two lines below
+# are valid code in both sh and perl.  When executed by sh, they re-execute
+# the script through the perl program found in $PATH.  The '-x' option
+# is essential as well; without it, perl would re-execute the script
+# through /bin/sh.  When executed by perl, the next two lines are a no-op.
+eval 'exec perl -wSx -pi "$0" "$@"'
+     if 0;
+
 s/a/b/
 EOF
 chmod a+x $s
@@ -54,7 +68,7 @@ echo a > $TMP-in
 # Skip this test if Perl is too old.  FIXME: 5.8.0 is just a guess.
 # We have a report that 5.6.1 is inadequate and that 5.8.0 works.
 perl -e 'require 5.8.0' || {
-  echo '$0: skipping this test; Perl version is too old' 1>&2
+  echo "$0: skipping this test; Perl version is too old" 1>&2
   exit 77
 }
 
@@ -67,152 +81,160 @@ export UPDATE_COPYRIGHT_MAX_LINE_LENGTH
 ## ----------------------------- ##
 
 TMP=$TMP_BASE-ex
-cat > $TMP.1 <<EOF
+cat > $TMP-1 <<EOF
 Copyright @copyright{} 1990-2005, 2007-2009 Free Software
 Foundation, Inc.
 EOF
-cat > $TMP.2 <<EOF
+cat > $TMP-2 <<EOF
 # Copyright (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
-cat > $TMP.3 <<EOF
+cat > $TMP-3 <<EOF
 /*
  * Copyright &copy; 90,2005,2007-2009
  * Free Software Foundation, Inc.
  */
 EOF
-cat > $TMP.4 <<EOF
+cat > $TMP-4 <<EOF
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 EOF
-cat > $TMP.5 <<EOF
+cat > $TMP-5 <<EOF
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 EOF
-cat > $TMP.6 <<EOF
+cat > $TMP-6 <<EOF
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 
 Copyright (C) 1990-2005, 2007-2009 Free Software Foundation,
 Inc.
 EOF
-cat > $TMP.7 <<EOF
+cat > $TMP-7 <<EOF
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 
 # Copyright (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
+cat > $TMP-8 <<EOF
+Copyright (C) 2008 Free Software Foundation, Inc.
+Copyright (C) 2008 Free Software Foundation, Inc.
+EOF
 
 UPDATE_COPYRIGHT_YEAR=2009 \
-  update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
+  update-copyright $TMP-? 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
 compare - $TMP-stderr <<EOF || exit 1
-$TMP.4: warning: copyright statement not found
-$TMP.5: warning: copyright statement not found
+$TMP-4: warning: copyright statement not found
+$TMP-5: warning: copyright statement not found
 EOF
-compare - $TMP.1 <<EOF || exit 1
+compare - $TMP-1 <<EOF || exit 1
 Copyright @copyright{} 1990-2005, 2007-2009 Free Software
 Foundation, Inc.
 EOF
-compare - $TMP.2 <<EOF || exit 1
+compare - $TMP-2 <<EOF || exit 1
 # Copyright (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
-compare - $TMP.3 <<EOF || exit 1
+compare - $TMP-3 <<EOF || exit 1
 /*
  * Copyright &copy; 90,2005,2007-2009
  * Free Software Foundation, Inc.
  */
 EOF
-compare - $TMP.4 <<EOF || exit 1
+compare - $TMP-4 <<EOF || exit 1
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 EOF
-compare - $TMP.5 <<EOF || exit 1
+compare - $TMP-5 <<EOF || exit 1
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 EOF
-compare - $TMP.6 <<EOF || exit 1
+compare - $TMP-6 <<EOF || exit 1
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 
 Copyright (C) 1990-2005, 2007-2009 Free Software Foundation,
 Inc.
 EOF
-compare - $TMP.7 <<EOF || exit 1
+compare - $TMP-7 <<EOF || exit 1
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 
 # Copyright (C) 1990-2005, 2007-2009 Free Software
 # Foundation, Inc.
 EOF
+compare - $TMP-8 <<EOF || exit 1
+Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+EOF
 
 UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
-  update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
+  update-copyright $TMP-? 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
 compare - $TMP-stderr <<EOF || exit 1
-$TMP.4: warning: copyright statement not found
-$TMP.5: warning: copyright statement not found
+$TMP-4: warning: copyright statement not found
+$TMP-5: warning: copyright statement not found
 EOF
-compare - $TMP.1 <<EOF || exit 1
+compare - $TMP-1 <<EOF || exit 1
 Copyright @copyright{} 1990-2005, 2007-2010 Free Software Foundation,
 Inc.
 EOF
-compare - $TMP.2 <<EOF || exit 1
+compare - $TMP-2 <<EOF || exit 1
 # Copyright (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
 EOF
-compare - $TMP.3 <<EOF || exit 1
+compare - $TMP-3 <<EOF || exit 1
 /*
  * Copyright &copy; 1990, 2005, 2007-2010 Free Software Foundation, Inc.
  */
 EOF
-compare - $TMP.4 <<EOF || exit 1
+compare - $TMP-4 <<EOF || exit 1
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 EOF
-compare - $TMP.5 <<EOF || exit 1
+compare - $TMP-5 <<EOF || exit 1
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 EOF
-compare - $TMP.6 <<EOF || exit 1
+compare - $TMP-6 <<EOF || exit 1
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 
 Copyright (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
 EOF
-compare - $TMP.7 <<EOF || exit 1
+compare - $TMP-7 <<EOF || exit 1
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 
 # Copyright (C) 1990-2005, 2007-2010 Free Software Foundation, Inc.
 EOF
 
 UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_FORCE=1 \
-  update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
+  update-copyright $TMP-? 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
 compare - $TMP-stderr <<EOF || exit 1
-$TMP.4: warning: copyright statement not found
-$TMP.5: warning: copyright statement not found
+$TMP-4: warning: copyright statement not found
+$TMP-5: warning: copyright statement not found
 EOF
-compare - $TMP.1 <<EOF || exit 1
+compare - $TMP-1 <<EOF || exit 1
 Copyright @copyright{} 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010
 Free Software Foundation, Inc.
 EOF
-compare - $TMP.2 <<EOF || exit 1
+compare - $TMP-2 <<EOF || exit 1
 # Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
 # 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
 # Software Foundation, Inc.
 EOF
-compare - $TMP.3 <<EOF || exit 1
+compare - $TMP-3 <<EOF || exit 1
 /*
  * Copyright &copy; 1990, 2005, 2007, 2008, 2009, 2010 Free Software
  * Foundation, Inc.
  */
 EOF
-compare - $TMP.4 <<EOF || exit 1
+compare - $TMP-4 <<EOF || exit 1
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 EOF
-compare - $TMP.5 <<EOF || exit 1
+compare - $TMP-5 <<EOF || exit 1
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 EOF
-compare - $TMP.6 <<EOF || exit 1
+compare - $TMP-6 <<EOF || exit 1
 ## Copyright (C) 1990-2005, 2007-2009 Free Software
 #  Foundation, Inc.
 
@@ -220,7 +242,7 @@ Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free
 Software Foundation, Inc.
 EOF
-compare - $TMP.7 <<EOF || exit 1
+compare - $TMP-7 <<EOF || exit 1
 Copyright (C) 1990-2005, 2007-2009 Acme, Inc.
 
 # Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
@@ -230,6 +252,43 @@ EOF
 
 rm $TMP*
 
+## ------------------------ ##
+## Examples from man pages  ##
+## ------------------------ ##
+
+TMP=$TMP_BASE-man
+cat > $TMP.1 <<EOF
+'\" Copyright (C) 1998-2009 Free Software Foundation, Inc.
+EOF
+cat > $TMP.2 <<EOF
+Copyright \(co 1998-2009 Free Software Foundation, Inc.
+EOF
+cat > $TMP.3 <<EOF
+Copyright \(co 1998\(en2009 Free Software Foundation, Inc.
+EOF
+cat > $TMP.4 <<EOF
+Copyright \(co 2009 Free Software Foundation, Inc.
+EOF
+
+UPDATE_COPYRIGHT_YEAR=2010 UPDATE_COPYRIGHT_USE_INTERVALS=1 \
+  update-copyright $TMP.? 1> $TMP-stdout 2> $TMP-stderr
+compare /dev/null $TMP-stdout || exit 1
+compare /dev/null $TMP-stderr || exit 1
+compare - $TMP.1 <<EOF || exit 1
+'\" Copyright (C) 1998\(en2010 Free Software Foundation, Inc.
+EOF
+compare - $TMP.2 <<EOF || exit 1
+Copyright \(co 1998\(en2010 Free Software Foundation, Inc.
+EOF
+compare - $TMP.3 <<EOF || exit 1
+Copyright \(co 1998\(en2010 Free Software Foundation, Inc.
+EOF
+compare - $TMP.4 <<EOF || exit 1
+Copyright \(co 2009\(en2010 Free Software Foundation, Inc.
+EOF
+
+rm $TMP*
+
 ## -------------- ##
 ## Current year.  ##
 ## -------------- ##
@@ -505,6 +564,9 @@ cat > $TMP.extra-text-space <<EOF
  /* Copyright 1987, 1988, 1991, 1992 Free Software Foundation, Inc. ***
     * End of comments. */
 EOF
+cat > $TMP.two-digit-final-is-substr-of-first <<EOF
+ /* Copyright 1991, 99 Free Software Foundation, Inc. */
+EOF
 UPDATE_COPYRIGHT_YEAR=2010 \
   update-copyright $TMP.* 1> $TMP-stdout 2> $TMP-stderr
 compare /dev/null $TMP-stdout || exit 1
@@ -536,6 +598,9 @@ compare - $TMP.extra-text-space <<EOF || exit 1
     Inc. ***
     * End of comments. */
 EOF
+compare - $TMP.two-digit-final-is-substr-of-first <<EOF || exit 1
+ /* Copyright 1991, 1999, 2010 Free Software Foundation, Inc. */
+EOF
 rm $TMP*
 
 exit 0
