@@ -1,6 +1,6 @@
 /* Host name canonicalization
 
-   Copyright (C) 2005, 2009-2022 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2009-2023 Free Software Foundation, Inc.
 
    Written by Derek Price <derek@ximbiot.com>
 
@@ -20,6 +20,11 @@
 #ifndef CANON_HOST_H
 # define CANON_HOST_H 1
 
+/* This file uses _GL_ATTRIBUTE_MALLOC.  */
+# if !_GL_CONFIG_H_INCLUDED
+#  error "Please include config.h first."
+# endif
+
 # include <stdlib.h>
 
 char *canon_host (char const *host)
