@@ -1,10 +1,10 @@
 /* settime -- set the system clock
 
-   Copyright (C) 2002, 2004-2007, 2009-2021 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004-2007, 2009-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -41,10 +41,8 @@ settime (struct timespec const *ts)
 
 #if HAVE_SETTIMEOFDAY
   {
-    struct timeval tv;
-
-    tv.tv_sec = ts->tv_sec;
-    tv.tv_usec = ts->tv_nsec / 1000;
+    struct timeval tv = { .tv_sec = ts->tv_sec,
+                          .tv_usec = ts->tv_nsec / 1000 };
     return settimeofday (&tv, 0);
   }
 #elif HAVE_STIME
