@@ -1,10 +1,10 @@
 /* Determine number of screen columns needed for a size-bounded wide string.
-   Copyright (C) 2011-2022 Free Software Foundation, Inc.
+   Copyright (C) 2011-2023 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2011.
 
    This file is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation, either version 3 of the
+   published by the Free Software Foundation; either version 2.1 of the
    License, or (at your option) any later version.
 
    This file is distributed in the hope that it will be useful,
@@ -22,4 +22,7 @@
 
 #include <limits.h>
 
+#define FUNC wcswidth
+#define UNIT wchar_t
+#define CHARACTER_WIDTH wcwidth
 #include "wcswidth-impl.h"
