@@ -1,5 +1,5 @@
 /* basename -- strip directory and suffix from file names
-   Copyright (C) 1990-2022 Free Software Foundation, Inc.
+   Copyright (C) 1990-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include <sys/types.h>
 
 #include "system.h"
-#include "error.h"
 #include "quote.h"
 
 /* The official name of this program (e.g., no 'g' prefix).  */
@@ -30,12 +29,12 @@
 
 static struct option const longopts[] =
 {
-  {"multiple", no_argument, NULL, 'a'},
-  {"suffix", required_argument, NULL, 's'},
-  {"zero", no_argument, NULL, 'z'},
+  {"multiple", no_argument, nullptr, 'a'},
+  {"suffix", required_argument, nullptr, 's'},
+  {"zero", no_argument, nullptr, 'z'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 void
@@ -97,7 +96,7 @@ remove_suffix (char *name, char const *suffix)
     *np = '\0';
 }
 
-/* Perform the basename operation on STRING.  If SUFFIX is non-NULL, remove
+/* Perform the basename operation on STRING.  If SUFFIX is non-null, remove
    the trailing SUFFIX.  Finally, output the result string.  */
 
 static void
@@ -125,7 +124,7 @@ main (int argc, char **argv)
 {
   bool multiple_names = false;
   bool use_nuls = false;
-  char const *suffix = NULL;
+  char const *suffix = nullptr;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -137,7 +136,7 @@ main (int argc, char **argv)
 
   while (true)
     {
-      int c = getopt_long (argc, argv, "+as:z", longopts, NULL);
+      int c = getopt_long (argc, argv, "+as:z", longopts, nullptr);
 
       if (c == -1)
         break;
@@ -184,7 +183,8 @@ main (int argc, char **argv)
     }
   else
     perform_basename (argv[optind],
-                      optind + 2 == argc ? argv[optind + 1] : NULL, use_nuls);
+                      optind + 2 == argc ? argv[optind + 1] : nullptr,
+                      use_nuls);
 
   return EXIT_SUCCESS;
 }
