@@ -24,10 +24,7 @@
 
 #include "system.h"
 #include "c-ctype.h"
-#include "die.h"
-#include "error.h"
 #include "fadvise.h"
-#include "idx.h"
 #include "quote.h"
 #include "xstrtol.h"
 #include "xdectoint.h"
@@ -50,7 +47,7 @@
 #elif BASE_TYPE == 42
 # include "base32.h"
 # include "base64.h"
-# include <assert.h>
+# include "assure.h"
 # define PROGRAM_NAME "basenc"
 #else
 # error missing/invalid BASE_TYPE definition
@@ -89,7 +86,7 @@ static struct option const long_options[] =
 #endif
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 void
@@ -449,7 +446,7 @@ base32hex_encode (char const *restrict in, idx_t inlen,
 
   for (char *p = out; outlen--; p++)
     {
-      assert (0x32 <= *p && *p <= 0x5a);          /* LCOV_EXCL_LINE */
+      affirm (0x32 <= *p && *p <= 0x5a);          /* LCOV_EXCL_LINE */
       *p = base32_norm_to_hex[*p - 0x32];
     }
 }
@@ -588,7 +585,7 @@ z85_length (int len)
 static bool
 isz85 (char ch)
 {
-  return c_isalnum (ch) || (strchr (".-:+=^!/*?&<>()[]{}@%$#", ch) != NULL);
+  return c_isalnum (ch) || strchr (".-:+=^!/*?&<>()[]{}@%$#", ch) != nullptr;
 }
 
 static char const z85_encoding[85] =
@@ -614,8 +611,8 @@ z85_encode (char const *restrict in, idx_t inlen,
             return;
 
           /* currently, there's no way to return an error in encoding.  */
-          die (EXIT_FAILURE, 0,
-               _("invalid input (length must be multiple of 4 characters)"));
+          error (EXIT_FAILURE, 0,
+                 _("invalid input (length must be multiple of 4 characters)"));
         }
       else
         {
@@ -926,7 +923,7 @@ wrap_write (char const *buffer, idx_t len,
     {
       /* Simple write. */
       if (fwrite (buffer, 1, len, stdout) < len)
-        die (EXIT_FAILURE, errno, _("write error"));
+        write_error ();
     }
   else
     for (idx_t written = 0; written < len; )
@@ -936,13 +933,13 @@ wrap_write (char const *buffer, idx_t len,
         if (to_write == 0)
           {
             if (fputc ('\n', out) == EOF)
-              die (EXIT_FAILURE, errno, _("write error"));
+              write_error ();
             *current_column = 0;
           }
         else
           {
             if (fwrite (buffer + written, 1, to_write, stdout) < to_write)
-              die (EXIT_FAILURE, errno, _("write error"));
+              write_error ();
             *current_column += to_write;
             written += to_write;
           }
@@ -955,9 +952,9 @@ finish_and_exit (FILE *in, char const *infile)
   if (fclose (in) != 0)
     {
       if (STREQ (infile, "-"))
-        die (EXIT_FAILURE, errno, _("closing standard input"));
+        error (EXIT_FAILURE, errno, _("closing standard input"));
       else
-        die (EXIT_FAILURE, errno, "%s", quotef (infile));
+        error (EXIT_FAILURE, errno, "%s", quotef (infile));
     }
 
   exit (EXIT_SUCCESS);
@@ -999,10 +996,10 @@ do_encode (FILE *in, char const *infile, FILE *out, idx_t wrap_column)
 
   /* When wrapping, terminate last line. */
   if (wrap_column && current_column > 0 && fputc ('\n', out) == EOF)
-    die (EXIT_FAILURE, errno, _("write error"));
+    write_error ();
 
   if (ferror (in))
-    die (EXIT_FAILURE, errno, _("read error"));
+    error (EXIT_FAILURE, errno, _("read error"));
 
   finish_and_exit (in, infile);
 }
@@ -1018,7 +1015,7 @@ do_decode (FILE *in, char const *infile, FILE *out, bool ignore_garbage)
   outbuf = xmalloc (DEC_BLOCKSIZE);
 
 #if BASE_TYPE == 42
-  ctx.inbuf = NULL;
+  ctx.inbuf = nullptr;
 #endif
   base_decode_ctx_init (&ctx);
 
@@ -1046,7 +1043,7 @@ do_decode (FILE *in, char const *infile, FILE *out, bool ignore_garbage)
           sum += n;
 
           if (ferror (in))
-            die (EXIT_FAILURE, errno, _("read error"));
+            error (EXIT_FAILURE, errno, _("read error"));
         }
       while (sum < BASE_LENGTH (DEC_BLOCKSIZE) && !feof (in));
 
@@ -1062,10 +1059,10 @@ do_decode (FILE *in, char const *infile, FILE *out, bool ignore_garbage)
           ok = base_decode_ctx (&ctx, inbuf, (k == 0 ? sum : 0), outbuf, &n);
 
           if (fwrite (outbuf, 1, n, out) < n)
-            die (EXIT_FAILURE, errno, _("write error"));
+            write_error ();
 
           if (!ok)
-            die (EXIT_FAILURE, 0, _("invalid input"));
+            error (EXIT_FAILURE, 0, _("invalid input"));
         }
     }
   while (!feof (in));
@@ -1099,7 +1096,7 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  while ((opt = getopt_long (argc, argv, "diw:", long_options, NULL)) != -1)
+  while ((opt = getopt_long (argc, argv, "diw:", long_options, nullptr)) != -1)
     switch (opt)
       {
       case 'd':
@@ -1109,10 +1106,10 @@ main (int argc, char **argv)
       case 'w':
         {
           intmax_t w;
-          strtol_error s_err = xstrtoimax (optarg, NULL, 10, &w, "");
+          strtol_error s_err = xstrtoimax (optarg, nullptr, 10, &w, "");
           if (LONGINT_OVERFLOW < s_err || w < 0)
-            die (EXIT_FAILURE, 0, "%s: %s",
-                 _("invalid wrap size"), quote (optarg));
+            error (EXIT_FAILURE, 0, "%s: %s",
+                   _("invalid wrap size"), quote (optarg));
           wrap_column = s_err == LONGINT_OVERFLOW || IDX_MAX < w ? 0 : w;
         }
         break;
@@ -1235,8 +1232,8 @@ main (int argc, char **argv)
   else
     {
       input_fh = fopen (infile, "rb");
-      if (input_fh == NULL)
-        die (EXIT_FAILURE, errno, "%s", quotef (infile));
+      if (input_fh == nullptr)
+        error (EXIT_FAILURE, errno, "%s", quotef (infile));
     }
 
   fadvise (input_fh, FADVISE_SEQUENTIAL);
