@@ -1,6 +1,6 @@
-/* expand-common - common functionality for expand/unexapnd
+/* expand-common - common functionality for expand/unexpand
 
-   Copyright (C) 1989-2022 Free Software Foundation, Inc.
+   Copyright (C) 1989-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,10 +52,10 @@ finalize_tab_stops (void);
 extern void
 set_file_list (char **file_list);
 
-/* Close the old stream pointer FP if it is non-NULL,
+/* Close the old stream pointer FP if it is non-null,
    and return a new one opened to read the next input file.
    Open a filename of '-' as the standard input.
-   Return NULL if there are no more input files.  */
+   Return nullptr if there are no more input files.  */
 extern FILE *
 next_file (FILE *fp);
 
