@@ -1,5 +1,5 @@
 /* mkfifo -- make fifo's (named pipes)
-   Copyright (C) 1990-2021 Free Software Foundation, Inc.
+   Copyright (C) 1990-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 #include <selinux/label.h>
 
 #include "system.h"
-#include "die.h"
-#include "error.h"
 #include "modechange.h"
 #include "quote.h"
 #include "selinux.h"
@@ -38,10 +36,10 @@
 static struct option const longopts[] =
 {
   {GETOPT_SELINUX_CONTEXT_OPTION_DECL},
-  {"mode", required_argument, NULL, 'm'},
+  {"mode", required_argument, nullptr, 'm'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 void
@@ -77,11 +75,11 @@ int
 main (int argc, char **argv)
 {
   mode_t newmode;
-  char const *specified_mode = NULL;
+  char const *specified_mode = nullptr;
   int exit_status = EXIT_SUCCESS;
   int optc;
-  char const *scontext = NULL;
-  struct selabel_handle *set_security_context = NULL;
+  char const *scontext = nullptr;
+  struct selabel_handle *set_security_context = nullptr;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -91,7 +89,7 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  while ((optc = getopt_long (argc, argv, "m:Z", longopts, NULL)) != -1)
+  while ((optc = getopt_long (argc, argv, "m:Z", longopts, nullptr)) != -1)
     {
       switch (optc)
         {
@@ -111,7 +109,7 @@ main (int argc, char **argv)
               else
                 {
                   set_security_context = selabel_open (SELABEL_CTX_FILE,
-                                                       NULL, 0);
+                                                       nullptr, 0);
                   if (! set_security_context)
                     error (0, errno, _("warning: ignoring --context"));
                 }
@@ -145,9 +143,9 @@ main (int argc, char **argv)
         ret = setfscreatecon (scontext);
 
       if (ret < 0)
-        die (EXIT_FAILURE, errno,
-             _("failed to set default file creation context to %s"),
-             quote (scontext));
+        error (EXIT_FAILURE, errno,
+               _("failed to set default file creation context to %s"),
+               quote (scontext));
     }
 
   newmode = MODE_RW_UGO;
@@ -156,14 +154,14 @@ main (int argc, char **argv)
       mode_t umask_value;
       struct mode_change *change = mode_compile (specified_mode);
       if (!change)
-        die (EXIT_FAILURE, 0, _("invalid mode"));
+        error (EXIT_FAILURE, 0, _("invalid mode"));
       umask_value = umask (0);
       umask (umask_value);
-      newmode = mode_adjust (newmode, false, umask_value, change, NULL);
+      newmode = mode_adjust (newmode, false, umask_value, change, nullptr);
       free (change);
       if (newmode & ~S_IRWXUGO)
-        die (EXIT_FAILURE, 0,
-             _("mode must specify only file permission bits"));
+        error (EXIT_FAILURE, 0,
+               _("mode must specify only file permission bits"));
     }
 
   for (; optind < argc; ++optind)
