@@ -23,14 +23,12 @@
 
 #include "system.h"
 #include "canonicalize.h"
-#include "die.h"
-#include "error.h"
 #include "relpath.h"
 
 /* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "realpath"
 
-#define AUTHORS proper_name ("Padraig Brady")
+#define AUTHORS proper_name_lite ("Padraig Brady", "P\303\241draig Brady")
 
 enum
 {
@@ -46,19 +44,19 @@ static char const *can_relative_base;
 
 static struct option const longopts[] =
 {
-  {"canonicalize-existing", no_argument, NULL, 'e'},
-  {"canonicalize-missing", no_argument, NULL, 'm'},
-  {"relative-to", required_argument, NULL, RELATIVE_TO_OPTION},
-  {"relative-base", required_argument, NULL, RELATIVE_BASE_OPTION},
-  {"quiet", no_argument, NULL, 'q'},
-  {"strip", no_argument, NULL, 's'},
-  {"no-symlinks", no_argument, NULL, 's'},
-  {"zero", no_argument, NULL, 'z'},
-  {"logical", no_argument, NULL, 'L'},
-  {"physical", no_argument, NULL, 'P'},
+  {"canonicalize-existing", no_argument, nullptr, 'e'},
+  {"canonicalize-missing", no_argument, nullptr, 'm'},
+  {"relative-to", required_argument, nullptr, RELATIVE_TO_OPTION},
+  {"relative-base", required_argument, nullptr, RELATIVE_BASE_OPTION},
+  {"quiet", no_argument, nullptr, 'q'},
+  {"strip", no_argument, nullptr, 's'},
+  {"no-symlinks", no_argument, nullptr, 's'},
+  {"zero", no_argument, nullptr, 'z'},
+  {"logical", no_argument, nullptr, 'L'},
+  {"physical", no_argument, nullptr, 'P'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 void
@@ -143,7 +141,7 @@ isdir (char const *path)
 {
   struct stat sb;
   if (stat (path, &sb) != 0)
-    die (EXIT_FAILURE, errno, _("cannot stat %s"), quoteaf (path));
+    error (EXIT_FAILURE, errno, _("cannot stat %s"), quoteaf (path));
   return S_ISDIR (sb.st_mode);
 }
 
@@ -160,7 +158,7 @@ process_path (char const *fname, int can_mode)
 
   if (!can_relative_to
       || (can_relative_base && !path_prefix (can_relative_base, can_fname))
-      || (can_relative_to && !relpath (can_fname, can_relative_to, NULL, 0)))
+      || (can_relative_to && !relpath (can_fname, can_relative_to, nullptr, 0)))
     fputs (can_fname, stdout);
 
   putchar (use_nuls ? '\0' : '\n');
@@ -175,8 +173,8 @@ main (int argc, char **argv)
 {
   bool ok = true;
   int can_mode = CAN_ALL_BUT_LAST;
-  char const *relative_to = NULL;
-  char const *relative_base = NULL;
+  char const *relative_to = nullptr;
+  char const *relative_base = nullptr;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -188,7 +186,7 @@ main (int argc, char **argv)
 
   while (true)
     {
-      int c = getopt_long (argc, argv, "eLmPqsz", longopts, NULL);
+      int c = getopt_long (argc, argv, "eLmPqsz", longopts, nullptr);
       if (c == -1)
         break;
       switch (c)
@@ -246,9 +244,9 @@ main (int argc, char **argv)
     {
       can_relative_to = realpath_canon (relative_to, can_mode);
       if (!can_relative_to)
-        die (EXIT_FAILURE, errno, "%s", quotef (relative_to));
+        error (EXIT_FAILURE, errno, "%s", quotef (relative_to));
       if (need_dir && !isdir (can_relative_to))
-        die (EXIT_FAILURE, ENOTDIR, "%s", quotef (relative_to));
+        error (EXIT_FAILURE, ENOTDIR, "%s", quotef (relative_to));
     }
   if (relative_base == relative_to)
     can_relative_base = can_relative_to;
@@ -256,9 +254,9 @@ main (int argc, char **argv)
     {
       char *base = realpath_canon (relative_base, can_mode);
       if (!base)
-        die (EXIT_FAILURE, errno, "%s", quotef (relative_base));
+        error (EXIT_FAILURE, errno, "%s", quotef (relative_base));
       if (need_dir && !isdir (base))
-        die (EXIT_FAILURE, ENOTDIR, "%s", quotef (relative_base));
+        error (EXIT_FAILURE, ENOTDIR, "%s", quotef (relative_base));
       /* --relative-to is a no-op if it does not have --relative-base
            as a prefix */
       if (path_prefix (base, can_relative_to))
@@ -267,7 +265,7 @@ main (int argc, char **argv)
         {
           free (base);
           can_relative_base = can_relative_to;
-          can_relative_to = NULL;
+          can_relative_to = nullptr;
         }
     }
 
