@@ -53,7 +53,7 @@ struct rm_options
   bool remove_empty_directories;
 
   /* Pointer to the device and inode numbers of '/', when --recursive
-     and preserving '/'.  Otherwise NULL.  */
+     and preserving '/'.  Otherwise null.  */
   struct dev_ino *root_dev_ino;
 
   /* If true, do not traverse into (or remove) any directory that is
