@@ -67,6 +67,7 @@
 # define makedev(maj, min)  mkdev (maj, min)
 #endif
 
+#include <stddef.h>
 #include <string.h>
 #include <errno.h>
 
@@ -191,7 +192,7 @@ select_plural (uintmax_t n)
 }
 
 #define STREQ(a, b) (strcmp (a, b) == 0)
-#define STREQ_LEN(a, b, n) (strncmp (a, b, n) == 0)
+#define STREQ_LEN(a, b, n) (strncmp (a, b, n) == 0) /* n==-1 means unbounded */
 #define STRPREFIX(a, b) (strncmp (a, b, strlen (b)) == 0)
 
 /* Just like strncmp, but the second argument must be a literal string
@@ -240,6 +241,7 @@ struct group *getgrgid (gid_t);
 uid_t getuid (void);
 #endif
 
+#include "idx.h"
 #include "xalloc.h"
 #include "verify.h"
 
@@ -282,7 +284,7 @@ readdir_ignoring_dot_and_dotdot (DIR *dirp)
   while (true)
     {
       struct dirent const *dp = readdir (dirp);
-      if (dp == NULL || ! dot_or_dotdot (dp->d_name))
+      if (dp == nullptr || ! dot_or_dotdot (dp->d_name))
         return dp;
     }
 }
@@ -310,7 +312,7 @@ directory_status (int fd_cwd, char const *dir)
     return errno;
 
   dirp = fdopendir (fd);
-  if (dirp == NULL)
+  if (dirp == nullptr)
     {
       saved_errno = errno;
       close (fd);
@@ -336,11 +338,11 @@ enum
 };
 
 #define GETOPT_HELP_OPTION_DECL \
-  "help", no_argument, NULL, GETOPT_HELP_CHAR
+  "help", no_argument, nullptr, GETOPT_HELP_CHAR
 #define GETOPT_VERSION_OPTION_DECL \
-  "version", no_argument, NULL, GETOPT_VERSION_CHAR
+  "version", no_argument, nullptr, GETOPT_VERSION_CHAR
 #define GETOPT_SELINUX_CONTEXT_OPTION_DECL \
-  "context", optional_argument, NULL, 'Z'
+  "context", optional_argument, nullptr, 'Z'
 
 #define case_GETOPT_HELP_CHAR			\
   case GETOPT_HELP_CHAR:			\
@@ -368,19 +370,18 @@ enum
 #undef emit_bug_reporting_address
 
 #include "propername.h"
-/* Define away proper_name (leaving proper_name_utf8, which affects far
-   fewer programs), since it's not worth the cost of adding ~17KB to
+/* Define away proper_name, since it's not worth the cost of adding ~17KB to
    the x86_64 text size of every single program.  This avoids a 40%
    (almost ~2MB) increase in the file system space utilization for the set
    of the 100 binaries. */
-#define proper_name(x) (x)
+#define proper_name(x) proper_name_lite (x, x)
 
 #include "progname.h"
 
 #define case_GETOPT_VERSION_CHAR(Program_name, Authors)			\
   case GETOPT_VERSION_CHAR:						\
     version_etc (stdout, Program_name, PACKAGE_NAME, Version, Authors,	\
-                 (char *) NULL);					\
+                 (char *) nullptr);					\
     exit (EXIT_SUCCESS);						\
     break;
 
@@ -565,7 +566,7 @@ is_nul (void const *buf, size_t length)
 
 #define DECIMAL_DIGIT_ACCUMULATE(Accum, Digit_val, Type)		\
   (									\
-   (void) (&(Accum) == (Type *) NULL),  /* The type matches.  */	\
+   (void) (&(Accum) == (Type *) nullptr),  /* The type matches.  */	\
    verify_expr (! TYPE_SIGNED (Type), /* The type is unsigned.  */      \
                 (((Type) -1 / 10 < (Accum)                              \
                   || (Type) ((Accum) * 10 + (Digit_val)) < (Accum))     \
@@ -663,7 +664,7 @@ emit_ancillary_info (char const *program)
     { "sha256sum", "sha2 utilities" },
     { "sha384sum", "sha2 utilities" },
     { "sha512sum", "sha2 utilities" },
-    { NULL, NULL }
+    { nullptr, nullptr }
   };
 
   char const *node = program;
@@ -679,7 +680,7 @@ emit_ancillary_info (char const *program)
 
   /* Don't output this redundant message for English locales.
      Note we still output for 'C' so that it gets included in the man page.  */
-  char const *lc_messages = setlocale (LC_MESSAGES, NULL);
+  char const *lc_messages = setlocale (LC_MESSAGES, nullptr);
   if (lc_messages && STRNCMP_LIT (lc_messages, "en_"))
     {
       /* TRANSLATORS: Replace LANG_CODE in this URL with your language code
@@ -737,6 +738,8 @@ usable_st_size (struct stat const *sb)
 
 _Noreturn void usage (int status);
 
+#include "error.h"
+
 /* Like error(0, 0, ...), but without an implicit newline.
    Also a noop unless the global DEV_DEBUG is set.  */
 #define devmsg(...)			\
@@ -759,6 +762,18 @@ The following directory is part of the cycle:\n  %s\n"), \
     }					\
   while (0)
 
+/* exit with a _single_ "write error" diagnostic.  */
+
+static inline void
+write_error (void)
+{
+  int saved_errno = errno;
+  fflush (stdout);    /* Last attempt to write any buffered data.  */
+  fpurge (stdout);    /* Ensure nothing buffered that might induce an error. */
+  clearerr (stdout);  /* Avoid extraneous diagnostic from close_stdout.  */
+  error (EXIT_FAILURE, saved_errno, _("write error"));
+}
+
 /* Like stpncpy, but do ensure that the result is NUL-terminated,
    and do not NUL-pad out to LEN.  I.e., when strnlen (src, len) == len,
    this function writes a NUL byte into dest[len].  Thus, the length
@@ -767,8 +782,8 @@ The following directory is part of the cycle:\n  %s\n"), \
 static inline char *
 stzncpy (char *restrict dest, char const *restrict src, size_t len)
 {
-  char const *src_end = src + len;
-  while (src < src_end && *src)
+  size_t i;
+  for (i = 0; i < len && *src; i++)
     *dest++ = *src++;
   *dest = 0;
   return dest;
