@@ -19,13 +19,11 @@
 #include <config.h>
 
 #include <stdio.h>
-#include <assert.h>
 #include <sys/types.h>
 #include <getopt.h>
 
 #include "system.h"
-#include "die.h"
-#include "error.h"
+#include "assure.h"
 #include "fadvise.h"
 #include "quote.h"
 #include "safe-read.h"
@@ -208,7 +206,7 @@ static bool delete = false;
 static bool complement = false;
 
 /* When tr is performing translation and string1 is longer than string2,
-   POSIX says that the result is unspecified.  That gives the implementor
+   POSIX says that the result is unspecified.  That gives the implementer
    of a POSIX conforming version of tr two reasonable choices for the
    semantics of this case.
 
@@ -268,13 +266,13 @@ static char xlate[N_CHARS];
 
 static struct option const long_options[] =
 {
-  {"complement", no_argument, NULL, 'c'},
-  {"delete", no_argument, NULL, 'd'},
-  {"squeeze-repeats", no_argument, NULL, 's'},
-  {"truncate-set1", no_argument, NULL, 't'},
+  {"complement", no_argument, nullptr, 'c'},
+  {"delete", no_argument, nullptr, 'd'},
+  {"squeeze-repeats", no_argument, nullptr, 's'},
+  {"truncate-set1", no_argument, nullptr, 't'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 void
@@ -408,7 +406,7 @@ is_char_class_member (enum Char_class char_class, unsigned char c)
       result = isxdigit (c);
       break;
     default:
-      abort ();
+      unreachable ();
     }
 
   return !! result;
@@ -591,7 +589,7 @@ make_printable_str (char const *s, size_t len)
   for (size_t i = 0; i < len; i++)
     {
       char buf[5];
-      char const *tmp = NULL;
+      char const *tmp = nullptr;
       unsigned char c = s[i];
 
       switch (c)
@@ -643,10 +641,9 @@ static void
 append_normal_char (struct Spec_list *list, unsigned char c)
 {
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = NULL;
+  new->next = nullptr;
   new->type = RE_NORMAL_CHAR;
   new->u.normal_char = c;
-  assert (list->tail);
   list->tail->next = new;
   list->tail = new;
 }
@@ -672,11 +669,10 @@ append_range (struct Spec_list *list, unsigned char first, unsigned char last)
       return false;
     }
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = NULL;
+  new->next = nullptr;
   new->type = RE_RANGE;
   new->u.range.first_char = first;
   new->u.range.last_char = last;
-  assert (list->tail);
   list->tail->next = new;
   list->tail = new;
   return true;
@@ -695,10 +691,9 @@ append_char_class (struct Spec_list *list,
   if (char_class == CC_NO_CLASS)
     return false;
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = NULL;
+  new->next = nullptr;
   new->type = RE_CHAR_CLASS;
   new->u.char_class = char_class;
-  assert (list->tail);
   list->tail->next = new;
   list->tail = new;
   return true;
@@ -714,11 +709,10 @@ append_repeated_char (struct Spec_list *list, unsigned char the_char,
                       count repeat_count)
 {
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = NULL;
+  new->next = nullptr;
   new->type = RE_REPEATED_CHAR;
   new->u.repeated_char.the_repeated_char = the_char;
   new->u.repeated_char.repeat_count = repeat_count;
-  assert (list->tail);
   list->tail->next = new;
   list->tail = new;
 }
@@ -737,10 +731,9 @@ append_equiv_class (struct Spec_list *list,
     return false;
 
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = NULL;
+  new->next = nullptr;
   new->type = RE_EQUIV_CLASS;
   new->u.equiv_code = *equiv_class_str;
-  assert (list->tail);
   list->tail->next = new;
   list->tail = new;
   return true;
@@ -781,7 +774,7 @@ find_bracketed_repeat (const struct E_string *es, size_t start_idx,
                        unsigned char *char_to_repeat, count *repeat_count,
                        size_t *closing_bracket_idx)
 {
-  assert (start_idx + 1 < es->len);
+  affirm (start_idx + 1 < es->len);
   if (!es_match (es, start_idx + 1, '*'))
     return -1;
 
@@ -804,7 +797,7 @@ find_bracketed_repeat (const struct E_string *es, size_t start_idx,
               char const *digit_str = &es->s[start_idx + 2];
               char *d_end;
               if ((xstrtoumax (digit_str, &d_end, *digit_str == '0' ? 8 : 10,
-                               repeat_count, NULL)
+                               repeat_count, nullptr)
                    != LONGINT_OK)
                   || REPEAT_COUNT_MAXIMUM < *repeat_count
                   || digit_str + digit_str_len != d_end)
@@ -992,7 +985,7 @@ build_spec_list (const struct E_string *es, struct Spec_list *result)
 }
 
 /* Advance past the current construct.
-   S->tail must be non-NULL.  */
+   S->tail must be non-null.  */
 static void
 skip_construct (struct Spec_list *s)
 {
@@ -1031,7 +1024,7 @@ get_next (struct Spec_list *s, enum Upper_Lower_class *class)
     }
 
   p = s->tail;
-  if (p == NULL)
+  if (p == nullptr)
     return -1;
 
   switch (p->type)
@@ -1076,10 +1069,10 @@ get_next (struct Spec_list *s, enum Upper_Lower_class *class)
           for (i = 0; i < N_CHARS; i++)
             if (is_char_class_member (p->u.char_class, i))
               break;
-          assert (i < N_CHARS);
+          affirm (i < N_CHARS);
           s->state = i;
         }
-      assert (is_char_class_member (p->u.char_class, s->state));
+      assure (is_char_class_member (p->u.char_class, s->state));
       return_val = s->state;
       for (i = s->state + 1; i < N_CHARS; i++)
         if (is_char_class_member (p->u.char_class, i))
@@ -1129,7 +1122,7 @@ get_next (struct Spec_list *s, enum Upper_Lower_class *class)
       break;
 
     default:
-      abort ();
+      unreachable ();
     }
 
   return return_val;
@@ -1148,7 +1141,7 @@ card_of_complement (struct Spec_list *s)
   bool in_set[N_CHARS] = { 0, };
 
   s->state = BEGIN_STATE;
-  while ((c = get_next (s, NULL)) != -1)
+  while ((c = get_next (s, nullptr)) != -1)
     {
       cardinality -= (!in_set[c]);
       in_set[c] = true;
@@ -1172,8 +1165,7 @@ validate_case_classes (struct Spec_list *s1, struct Spec_list *s2)
   size_t n_lower = 0;
   int c1 = 0;
   int c2 = 0;
-  count old_s1_len = s1->length;
-  count old_s2_len = s2->length;
+  MAYBE_UNUSED count old_s1_len = s1->length, old_s2_len = s2->length;
   struct List_element *s1_tail = s1->tail;
   struct List_element *s2_tail = s2->tail;
   bool s1_new_element = true;
@@ -1204,8 +1196,8 @@ validate_case_classes (struct Spec_list *s1, struct Spec_list *s2)
          c1 must also transition at the same time.  */
       if (s2_new_element && class_s2 != UL_NONE
           && !(s1_new_element && class_s1 != UL_NONE))
-        die (EXIT_FAILURE, 0,
-             _("misaligned [:upper:] and/or [:lower:] construct"));
+        error (EXIT_FAILURE, 0,
+               _("misaligned [:upper:] and/or [:lower:] construct"));
 
       /* If case converting, quickly skip over the elements.  */
       if (class_s2 != UL_NONE)
@@ -1221,7 +1213,7 @@ validate_case_classes (struct Spec_list *s1, struct Spec_list *s2)
       s2_new_element = s2->state == NEW_ELEMENT; /* Next element is new.  */
     }
 
-  assert (old_s1_len >= s1->length && old_s2_len >= s2->length);
+  affirm (old_s1_len >= s1->length && old_s2_len >= s2->length);
 
   s1->tail = s1_tail;
   s2->tail = s2_tail;
@@ -1262,7 +1254,7 @@ get_spec_stats (struct Spec_list *s)
           break;
 
         case RE_RANGE:
-          assert (p->u.range.last_char >= p->u.range.first_char);
+          affirm (p->u.range.last_char >= p->u.range.first_char);
           len = p->u.range.last_char - p->u.range.first_char + 1;
           break;
 
@@ -1300,7 +1292,7 @@ get_spec_stats (struct Spec_list *s)
           break;
 
         default:
-          abort ();
+          unreachable ();
         }
 
       /* Check for arithmetic overflow in computing length.  Also, reject
@@ -1309,7 +1301,7 @@ get_spec_stats (struct Spec_list *s)
          indefinite element.  */
       new_length = length + len;
       if (! (length <= new_length && new_length <= REPEAT_COUNT_MAXIMUM))
-        die (EXIT_FAILURE, 0, _("too many characters in set"));
+        error (EXIT_FAILURE, 0, _("too many characters in set"));
       length = new_length;
     }
 
@@ -1341,7 +1333,7 @@ spec_init (struct Spec_list *spec_list)
 {
   struct List_element *new = xmalloc (sizeof *new);
   spec_list->head = spec_list->tail = new;
-  spec_list->head->next = NULL;
+  spec_list->head->next = nullptr;
 }
 
 /* This function makes two passes over the argument string S.  The first
@@ -1374,9 +1366,9 @@ string2_extend (const struct Spec_list *s1, struct Spec_list *s2)
   struct List_element *p;
   unsigned char char_to_repeat;
 
-  assert (translating);
-  assert (s1->length > s2->length);
-  assert (s2->length > 0);
+  affirm (translating);
+  affirm (s1->length > s2->length);
+  affirm (s2->length > 0);
 
   p = s2->tail;
   switch (p->type)
@@ -1392,9 +1384,9 @@ string2_extend (const struct Spec_list *s1, struct Spec_list *s2)
            tr '[:upper:]0-9' '[:lower:]'
          That's not portable however, contradicts POSIX and is dependent
          on your collating sequence.  */
-      die (EXIT_FAILURE, 0,
-           _("when translating with string1 longer than string2,\nthe\
- latter string must not end with a character class"));
+      error (EXIT_FAILURE, 0,
+             _("when translating with string1 longer than string2,\n"
+               "the latter string must not end with a character class"));
 
     case RE_REPEATED_CHAR:
       char_to_repeat = p->u.repeated_char.the_repeated_char;
@@ -1403,10 +1395,10 @@ string2_extend (const struct Spec_list *s1, struct Spec_list *s2)
     case RE_EQUIV_CLASS:
       /* This shouldn't happen, because validate exits with an error
          if it finds an equiv class in string2 when translating.  */
-      abort ();
+      affirm (false);
 
     default:
-      abort ();
+      unreachable ();
     }
 
   append_repeated_char (s2, char_to_repeat, s1->length - s2->length);
@@ -1424,10 +1416,10 @@ homogeneous_spec_list (struct Spec_list *s)
 
   s->state = BEGIN_STATE;
 
-  if ((b = get_next (s, NULL)) == -1)
+  if ((b = get_next (s, nullptr)) == -1)
     return false;
 
-  while ((c = get_next (s, NULL)) != -1)
+  while ((c = get_next (s, nullptr)) != -1)
     if (c != b)
       return false;
 
@@ -1447,36 +1439,29 @@ validate (struct Spec_list *s1, struct Spec_list *s2)
 {
   get_s1_spec_stats (s1);
   if (s1->n_indefinite_repeats > 0)
-    {
-      die (EXIT_FAILURE, 0,
+    error (EXIT_FAILURE, 0,
            _("the [c*] repeat construct may not appear in string1"));
-    }
 
   if (s2)
     {
       get_s2_spec_stats (s2, s1->length);
 
       if (s2->n_indefinite_repeats > 1)
-        {
-          die (EXIT_FAILURE, 0,
+        error (EXIT_FAILURE, 0,
                _("only one [c*] repeat construct may appear in string2"));
-        }
 
       if (translating)
         {
           if (s2->has_equiv_class)
-            {
-              die (EXIT_FAILURE, 0,
-                   _("[=c=] expressions may not appear in string2\
- when translating"));
-            }
+            error (EXIT_FAILURE, 0,
+                   _("[=c=] expressions may not appear in string2"
+                     " when translating"));
 
           if (s2->has_restricted_char_class)
-            {
-              die (EXIT_FAILURE, 0,
-                   _("when translating, the only character classes that may\
- appear in\nstring2 are 'upper' and 'lower'"));
-            }
+            error (EXIT_FAILURE, 0,
+                   _("when translating, the only character classes"
+                     " that may appear in\n"
+                     "string2 are 'upper' and 'lower'"));
 
           validate_case_classes (s1, s2);
 
@@ -1488,33 +1473,32 @@ validate (struct Spec_list *s1, struct Spec_list *s2)
                      given or string1 is empty.  */
 
                   if (s2->length == 0)
-                    die (EXIT_FAILURE, 0,
-                     _("when not truncating set1, string2 must be non-empty"));
+                    error (EXIT_FAILURE, 0,
+                           _("when not truncating set1,"
+                             " string2 must be non-empty"));
                   string2_extend (s1, s2);
                 }
             }
 
           if (complement && s1->has_char_class
               && ! (s2->length == s1->length && homogeneous_spec_list (s2)))
-            {
-              die (EXIT_FAILURE, 0,
-                   _("when translating with complemented character classes,\
-\nstring2 must map all characters in the domain to one"));
-            }
+            error (EXIT_FAILURE, 0,
+                   _("when translating with complemented character classes,\n"
+                     "string2 must map all characters in the domain to one"));
         }
       else
         /* Not translating.  */
         {
           if (s2->n_indefinite_repeats > 0)
-            die (EXIT_FAILURE, 0,
-                 _("the [c*] construct may appear in string2 only\
- when translating"));
+            error (EXIT_FAILURE, 0,
+                   _("the [c*] construct may appear in string2"
+                     " only when translating"));
         }
     }
 }
 
 /* Read buffers of SIZE bytes via the function READER (if READER is
-   NULL, read from stdin) until EOF.  When non-NULL, READER is either
+   null, read from stdin) until EOF.  When non-null, READER is either
    read_and_delete or read_and_xlate.  After each buffer is read, it is
    processed and written to stdout.  The buffers are processed so that
    multiple consecutive occurrences of the same character in the input
@@ -1587,7 +1571,7 @@ squeeze_filter (char *buf, size_t size, size_t (*reader) (char *, size_t))
             }
           if (out_len > 0
               && fwrite (&buf[begin], 1, out_len, stdout) != out_len)
-            die (EXIT_FAILURE, errno, _("write error"));
+            write_error ();
         }
 
       if (char_to_squeeze != NOT_A_CHAR)
@@ -1611,7 +1595,7 @@ plain_read (char *buf, size_t size)
 {
   size_t nr = safe_read (STDIN_FILENO, buf, size);
   if (nr == SAFE_READ_ERROR)
-    die (EXIT_FAILURE, errno, _("read error"));
+    error (EXIT_FAILURE, errno, _("read error"));
   return nr;
 }
 
@@ -1682,7 +1666,7 @@ set_initialize (struct Spec_list *s, bool complement_this_set, bool *in_set)
   int c;
 
   s->state = BEGIN_STATE;
-  while ((c = get_next (s, NULL)) != -1)
+  while ((c = get_next (s, nullptr)) != -1)
     in_set[c] = true;
   if (complement_this_set)
     for (size_t i = 0; i < N_CHARS; i++)
@@ -1708,7 +1692,7 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  while ((c = getopt_long (argc, argv, "+AcCdst", long_options, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, "+AcCdst", long_options, nullptr)) != -1)
     {
       switch (c)
         {
@@ -1787,7 +1771,7 @@ main (int argc, char **argv)
         main_exit (EXIT_FAILURE);
     }
   else
-    s2 = NULL;
+    s2 = nullptr;
 
   validate (s1, s2);
 
@@ -1813,7 +1797,7 @@ main (int argc, char **argv)
           if (nr == 0)
             break;
           if (fwrite (io_buf, 1, nr, stdout) != nr)
-            die (EXIT_FAILURE, errno, _("write error"));
+            write_error ();
         }
     }
   else if (squeeze_repeats && delete && non_option_args == 2)
@@ -1836,8 +1820,8 @@ main (int argc, char **argv)
             {
               if (!in_s1[i])
                 {
-                  int ch = get_next (s2, NULL);
-                  assert (ch != -1 || truncate_set1);
+                  int ch = get_next (s2, nullptr);
+                  affirm (ch != -1 || truncate_set1);
                   if (ch == -1)
                     {
                       /* This will happen when tr is invoked like e.g.
@@ -1890,7 +1874,7 @@ main (int argc, char **argv)
                   skip_construct (s2);
                 }
             }
-          assert (c1 == -1 || truncate_set1);
+          affirm (c1 == -1 || truncate_set1);
         }
       if (squeeze_repeats)
         {
@@ -1905,13 +1889,13 @@ main (int argc, char **argv)
               if (bytes_read == 0)
                 break;
               if (fwrite (io_buf, 1, bytes_read, stdout) != bytes_read)
-                die (EXIT_FAILURE, errno, _("write error"));
+                write_error ();
             }
         }
     }
 
   if (close (STDIN_FILENO) != 0)
-    die (EXIT_FAILURE, errno, _("standard input"));
+    error (EXIT_FAILURE, errno, _("standard input"));
 
   main_exit (EXIT_SUCCESS);
 }
