@@ -73,7 +73,7 @@ main (int argc, char **argv)
 
       if (STREQ (argv[1], "--version"))
         version_etc (stdout, PROGRAM_NAME, PACKAGE_NAME, Version, AUTHORS,
-                     (char *) NULL);
+                     (char *) nullptr);
     }
 
   return EXIT_STATUS;
