@@ -29,13 +29,12 @@ echo n | returns_ 1 cp -iR a b 2>/dev/null || fail=1
 # test miscellaneous combinations of -f -i -n parameters
 touch c d || framework_failure_
 echo "'c' -> 'd'" > out_copy || framework_failure_
-echo "skipped 'd'" > out_skip || framework_failure_
 echo "cp: not replacing 'd'" > err_skip || framework_failure_
 touch out_empty || framework_failure_
 
 # ask for overwrite, answer no
 echo n | returns_ 1 cp -vi  c d 2>/dev/null > out1 || fail=1
-compare out1 out_skip || fail=1
+compare out1 out_empty || fail=1
 
 # ask for overwrite, answer yes
 echo y | cp -vi  c d 2>/dev/null > out2 || fail=1
@@ -47,7 +46,7 @@ compare out3 out_copy  || fail=1
 
 # -n wins over -i
 echo y | returns_ 1 cp -vin c d 2>/dev/null > out4 || fail=1
-compare out4 out_skip || fail=1
+compare out4 out_empty || fail=1
 
 # -n wins over -i non verbose
 echo y | returns_ 1 cp -in c d 2>err4 > out4 || fail=1
@@ -60,11 +59,11 @@ compare out5 out_copy  || fail=1
 
 # do not ask, prevent from overwrite
 echo n | returns_ 1 cp -vfn c d 2>/dev/null > out6 || fail=1
-compare out6 out_skip || fail=1
+compare out6 out_empty || fail=1
 
 # do not ask, prevent from overwrite
 echo n | returns_ 1 cp -vnf c d 2>/dev/null > out7 || fail=1
-compare out7 out_skip || fail=1
+compare out7 out_empty || fail=1
 
 # options --backup and --no-clobber are mutually exclusive
 returns_ 1 cp -bn c d 2>/dev/null || fail=1
