@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 # Test "date".
 
-# Copyright (C) 2005-2026 Free Software Foundation, Inc.
+# Copyright (C) 2005-2023 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -185,9 +185,6 @@ my @Tests =
      ['relative-3', "-I -d '2006-04-23 21 days ago'", {OUT=>"2006-04-02"},
          {ENV=>'TZ=PST8PDT,M4.1.0,M10.5.0'}],
 
-     # Relative days, negative not confused with option
-     ['relative-4', "--utc -d '-1 day' +%Z", {OUT=>"UTC"}],
-
      # This would infloop (or appear to) prior to coreutils-4.5.5,
      # due to a bug in strftime.c.
      ['wide-fmt', "-d '1999-06-01'", '+%3004Y', {OUT=>'0' x 3000 . "1999"}],
@@ -319,26 +316,6 @@ my @Tests =
 
      # test with %%-N
      ['pct-pct', '+%%-N', {OUT => '%-N'}],
-
-     # Test parenthesis comment handling
-     # Single parenthesis - should be treated as empty string (midnight today)
-     ['paren-1', "-d '(' +'%H:%M:%S'", {OUT=>"00:00:00"}],
-
-     # Parenthesis with preceding text - comment should be ignored
-     ['paren-2', "-d '1(ignore this comment' +'%H:%M:%S'", {OUT=>"01:00:00"}],
-
-     # Parenthesis with date - comment should be ignored
-     ['paren-3', "-d '2026-01-05(this is a comment' -u +'%Y-%m-%d'",
-      {OUT=>"2026-01-05"}],
-
-     # Text enclosed in parentheses is treated as a comment
-     ['paren-4', "-d '2026(this is a comment)-01-05' -u +'%Y-%m-%d'",
-      {OUT=>"2026-01-05"}],
-
-     # Nested comments are supported
-     ['paren-5', "-d '((nested))2026-01-05' -u +'%Y-%m-%d'",
-      {OUT=>"2026-01-05"}],
-     ['paren-6', "-d '((nested)2026-01-05)' +'%H:%M:%S'", {OUT=>"00:00:00"}],
     );
 
 # Repeat the cross-dst test, using Jan 1, 2005 and every interval from 1..364.
