@@ -35,18 +35,6 @@ split -n l/10 /dev/null || fail=1
 test "$(stat -c %s x* | uniq -c | sed 's/^ *//; s/ /x/')" = "10x0" || fail=1
 rm x??
 
-# 'split' should reject any attempt to create an infinitely
-# long output file.
-# This test is very expensive as it runs out of /tmp space.
-if test "${RUN_VERY_EXPENSIVE_TESTS+set}" = set; then
-  returns_ 1 split -n l/2 /dev/zero || fail=1
-  rm x??
-
-  # Repeat the above,  but with 1/2, not l/2:
-  returns_ 1 split -n 1/2 /dev/zero || fail=1
-  rm x??
-fi
-
 # Ensure --elide-empty-files is honored
 split -e -n l/10 /dev/null || fail=1
 returns_ 1 stat x?? 2>/dev/null || fail=1
@@ -72,12 +60,11 @@ printf '%s' "\
 
 sed 's/00 *//g' exp > exp.elide_empty || framework_failure_
 
-DEBUGGING=
-test "$DEBUGGING" && test "$VERBOSE" && set +x
+test "$DEBUG" && test "$VERBOSE" && set +x
 for ELIDE_EMPTY in '' '-e'; do
   for IO_BLKSIZE in 1 2 5 10 80 100; do
     > out
-    test "$DEBUGGING" && printf "\n---io-blk-size=$IO_BLKSIZE $ELIDE_EMPTY\n"
+    test "$DEBUG" && printf "\n---io-blk-size=$IO_BLKSIZE $ELIDE_EMPTY\n"
     for N in 6 8 12 15 22; do
       rm -f x*
 
@@ -93,7 +80,7 @@ for ELIDE_EMPTY in '' '-e'; do
         compare chunk.k xab || fail=1
       fi
 
-      if test "$DEBUGGING"; then
+      if test "$DEBUG"; then
         # Output partition pattern
         size=$(printf "%s" "$lines" | wc -c)
         chunk_size=$(($size/$N))
@@ -120,7 +107,7 @@ for ELIDE_EMPTY in '' '-e'; do
     compare out $EXP || fail=1
   done
 done
-test "$DEBUGGING" && test "$VERBOSE" && set -x
+test "$DEBUG" && test "$VERBOSE" && set -x
 
 
 # Check extraction of particular chunks
